# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import platform
import sys
from datetime import datetime

from urllib.parse import quote
from ..version import __version__
from ..qt import QtCore, QtNetwork, QtWidgets
from ..local_config import LocalConfig
from ..settings import GENERAL_SETTINGS

import logging
log = logging.getLogger(__name__)


class AnalyticsClient(QtCore.QObject):
    """
    Google analytics client to send events.
    """

    _property_id = "UA-55817127-3"

    def __init__(self):
        super().__init__()
        self._visitor_id = None
        self._manager = QtNetwork.QNetworkAccessManager(self)

        def finished(network_reply):
            if network_reply.error() != QtNetwork.QNetworkReply.NoError:
                log.debug("Error when pushing to Google Analytics %s", network_reply.errorString())

        self._manager.finished.connect(finished)

        #
        # We need to build a user agent for Universal Analytics in order to
        # let analytics guess the OS
        # this could break by analytics at anytime :(
        if sys.platform.startswith("darwin"):
            self._user_agent = "Mozilla/5.0 (Macintosh; Intel Mac OS X {release}) AppleWebKit/537.36 (KHTML, like Gecko) GNS3/{version}".format(release=platform.mac_ver()[0].replace(".", "_"), version=__version__)
        elif sys.platform.startswith("win"):
            self._user_agent = "Mozilla/5.0 (Windows NT {release}) AppleWebKit/537.36 (KHTML, like Gecko) Safari/537.36 GNS3/{version}".format(release=platform.release(), version=__version__)
        else:
            self._user_agent = "Mozilla/5.0 (X11; Linux {arch}) AppleWebKit/537.36 (KHTML, like Gecko) Safari/537.36  GNS3/{version}".format(arch=platform.machine(), version=__version__)
        self._rate_limit = {}

    def sendScreenView(self, screen, session_start=None):
        """
        :params session_start: True session start, None during session, False session stop
        """

        if session_start is not False and screen in self._rate_limit:
            if self._rate_limit[screen] + 60 * 1 > datetime.utcnow().timestamp():
                log.debug("Ignore call %s to Google Analytics because of rate limiting", screen)
                return

        self._rate_limit[screen] = datetime.utcnow().timestamp()

        settings = LocalConfig.instance().loadSectionSettings("MainWindow", GENERAL_SETTINGS)
        if settings["send_stats"] is False:
            log.debug("Stats is turn off ignore call %s", screen)
            return

        body = "v=1"  # Version
        body += "&tid={}".format(self._property_id)  # Tracking ID / Property ID
        body += "&cid={}".format(settings["stats_visitor_id"])  # Anonymous Client ID
        body += "&aip=1"  # Anonymize IP
        body += "&t=screenview"  # Screenview hit type
        body += "&an=GNS3"  # App name
        body += "&av={}".format(quote(__version__))  # App version.
        body += "&ua={}".format(quote(self._user_agent))  # User agent
        body += "&cd={}".format(quote(screen))  # Category
        body += "&ds=gns3-gui"  # Data source
        if session_start is True:
            body += "&sc=start"  # Session start
        elif session_start is False:
            body += "&sc=end"  # Session end

        screen = QtWidgets.QApplication.desktop().screenGeometry()
        body += "&sr={}x{}".format(screen.width(), screen.height())  # Screen resolution

        locale = QtCore.QLocale.system().name().lower()
        if locale:
            body += "&ul={}".format(locale)  # User language

        # TODO: HTTPS when possible because it's broken for the moment with Qt on OSX:
        # https://bugreports.qt.io/browse/QTBUG-45487
        if sys.platform.startswith("darwin"):
            url = QtCore.QUrl('http://www.google-analytics.com/collect')
        else:
            url = QtCore.QUrl('https://www.google-analytics.com/collect')
        request_qt = QtNetwork.QNetworkRequest(url)
        request_qt.setRawHeader(b"Content-Type", b"application/x-www-form-urlencoded")
        request_qt.setRawHeader(b"User-Agent", self._user_agent.encode())
        self._manager.post(request_qt, body.encode())

        log.debug("Send stats to Google Analytics: %s", body)

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of AnalyticsClient.
        :returns: instance of AnalyticsClient
        """

        if not hasattr(AnalyticsClient, '_instance') or AnalyticsClient._instance is None:
            AnalyticsClient._instance = AnalyticsClient()
        return AnalyticsClient._instance



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/ui/general_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.5.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_GeneralPreferencesPageWidget(object):
    def setupUi(self, GeneralPreferencesPageWidget):
        GeneralPreferencesPageWidget.setObjectName("GeneralPreferencesPageWidget")
        GeneralPreferencesPageWidget.resize(538, 623)
        self.verticalLayout = QtWidgets.QVBoxLayout(GeneralPreferencesPageWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.uiMiscTabWidget = QtWidgets.QTabWidget(GeneralPreferencesPageWidget)
        self.uiMiscTabWidget.setObjectName("uiMiscTabWidget")
        self.uiGeneralTab = QtWidgets.QWidget()
        self.uiGeneralTab.setObjectName("uiGeneralTab")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.uiGeneralTab)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.uiLocalPathsGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalPathsGroupBox.sizePolicy().hasHeightForWidth())
        self.uiLocalPathsGroupBox.setSizePolicy(sizePolicy)
        self.uiLocalPathsGroupBox.setObjectName("uiLocalPathsGroupBox")
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.uiLocalPathsGroupBox)
        self.verticalLayout_7.setObjectName("verticalLayout_7")
        self.uiProjectsPathLabel = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.uiProjectsPathLabel.setObjectName("uiProjectsPathLabel")
        self.verticalLayout_7.addWidget(self.uiProjectsPathLabel)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.uiProjectsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiProjectsPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiProjectsPathLineEdit.setSizePolicy(sizePolicy)
        self.uiProjectsPathLineEdit.setObjectName("uiProjectsPathLineEdit")
        self.horizontalLayout_2.addWidget(self.uiProjectsPathLineEdit)
        self.uiProjectsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiProjectsPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiProjectsPathToolButton.setObjectName("uiProjectsPathToolButton")
        self.horizontalLayout_2.addWidget(self.uiProjectsPathToolButton)
        self.verticalLayout_7.addLayout(self.horizontalLayout_2)
        self.uiImagesPathLabel = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.uiImagesPathLabel.setObjectName("uiImagesPathLabel")
        self.verticalLayout_7.addWidget(self.uiImagesPathLabel)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.uiImagesPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiImagesPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiImagesPathLineEdit.setSizePolicy(sizePolicy)
        self.uiImagesPathLineEdit.setObjectName("uiImagesPathLineEdit")
        self.horizontalLayout_4.addWidget(self.uiImagesPathLineEdit)
        self.uiImagesPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiImagesPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiImagesPathToolButton.setObjectName("uiImagesPathToolButton")
        self.horizontalLayout_4.addWidget(self.uiImagesPathToolButton)
        self.verticalLayout_7.addLayout(self.horizontalLayout_4)
        self.label = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.label.setObjectName("label")
        self.verticalLayout_7.addWidget(self.label)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.uiSymbolsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        self.uiSymbolsPathLineEdit.setObjectName("uiSymbolsPathLineEdit")
        self.horizontalLayout_3.addWidget(self.uiSymbolsPathLineEdit)
        self.uiSymbolsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiSymbolsPathToolButton.setObjectName("uiSymbolsPathToolButton")
        self.horizontalLayout_3.addWidget(self.uiSymbolsPathToolButton)
        self.verticalLayout_7.addLayout(self.horizontalLayout_3)
        self.uiConfigsPathLabel = QtWidgets.QLabel(self.uiLocalPathsGroupBox)
        self.uiConfigsPathLabel.setObjectName("uiConfigsPathLabel")
        self.verticalLayout_7.addWidget(self.uiConfigsPathLabel)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.uiConfigsPathLineEdit = QtWidgets.QLineEdit(self.uiLocalPathsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConfigsPathLineEdit.sizePolicy().hasHeightForWidth())
        self.uiConfigsPathLineEdit.setSizePolicy(sizePolicy)
        self.uiConfigsPathLineEdit.setObjectName("uiConfigsPathLineEdit")
        self.horizontalLayout_7.addWidget(self.uiConfigsPathLineEdit)
        self.uiConfigsPathToolButton = QtWidgets.QToolButton(self.uiLocalPathsGroupBox)
        self.uiConfigsPathToolButton.setToolButtonStyle(QtCore.Qt.ToolButtonTextOnly)
        self.uiConfigsPathToolButton.setObjectName("uiConfigsPathToolButton")
        self.horizontalLayout_7.addWidget(self.uiConfigsPathToolButton)
        self.verticalLayout_7.addLayout(self.horizontalLayout_7)
        self.verticalLayout_5.addWidget(self.uiLocalPathsGroupBox)
        self.uiStyleGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiStyleGroupBox.sizePolicy().hasHeightForWidth())
        self.uiStyleGroupBox.setSizePolicy(sizePolicy)
        self.uiStyleGroupBox.setObjectName("uiStyleGroupBox")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.uiStyleGroupBox)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.uiStyleComboBox = QtWidgets.QComboBox(self.uiStyleGroupBox)
        self.uiStyleComboBox.setObjectName("uiStyleComboBox")
        self.verticalLayout_4.addWidget(self.uiStyleComboBox)
        self.verticalLayout_5.addWidget(self.uiStyleGroupBox)
        self.uiConfigurationFileGroupBox = QtWidgets.QGroupBox(self.uiGeneralTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiConfigurationFileGroupBox.sizePolicy().hasHeightForWidth())
        self.uiConfigurationFileGroupBox.setSizePolicy(sizePolicy)
        self.uiConfigurationFileGroupBox.setObjectName("uiConfigurationFileGroupBox")
        self.gridLayout = QtWidgets.QGridLayout(self.uiConfigurationFileGroupBox)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.uiImportConfigurationFilePushButton = QtWidgets.QPushButton(self.uiConfigurationFileGroupBox)
        self.uiImportConfigurationFilePushButton.setObjectName("uiImportConfigurationFilePushButton")
        self.horizontalLayout.addWidget(self.uiImportConfigurationFilePushButton)
        self.uiExportConfigurationFilePushButton = QtWidgets.QPushButton(self.uiConfigurationFileGroupBox)
        self.uiExportConfigurationFilePushButton.setObjectName("uiExportConfigurationFilePushButton")
        self.horizontalLayout.addWidget(self.uiExportConfigurationFilePushButton)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.gridLayout.addLayout(self.horizontalLayout, 1, 0, 1, 1)
        self.uiConfigurationFileLabel = QtWidgets.QLabel(self.uiConfigurationFileGroupBox)
        self.uiConfigurationFileLabel.setObjectName("uiConfigurationFileLabel")
        self.gridLayout.addWidget(self.uiConfigurationFileLabel, 0, 0, 1, 1)
        self.verticalLayout_5.addWidget(self.uiConfigurationFileGroupBox)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem1)
        self.uiMiscTabWidget.addTab(self.uiGeneralTab, "")
        self.uiConsoleTab = QtWidgets.QWidget()
        self.uiConsoleTab.setObjectName("uiConsoleTab")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.uiConsoleTab)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.uiTelnetConsoleSettingsGroupBox = QtWidgets.QGroupBox(self.uiConsoleTab)
        self.uiTelnetConsoleSettingsGroupBox.setObjectName("uiTelnetConsoleSettingsGroupBox")
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.uiTelnetConsoleSettingsGroupBox)
        self.verticalLayout_9.setObjectName("verticalLayout_9")
        self.uiTelnetConsoleCommandLabel = QtWidgets.QLabel(self.uiTelnetConsoleSettingsGroupBox)
        self.uiTelnetConsoleCommandLabel.setObjectName("uiTelnetConsoleCommandLabel")
        self.verticalLayout_9.addWidget(self.uiTelnetConsoleCommandLabel)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.uiTelnetConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTelnetConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiTelnetConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiTelnetConsoleCommandLineEdit.setReadOnly(True)
        self.uiTelnetConsoleCommandLineEdit.setObjectName("uiTelnetConsoleCommandLineEdit")
        self.horizontalLayout_9.addWidget(self.uiTelnetConsoleCommandLineEdit)
        self.uiTelnetConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiTelnetConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiTelnetConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiTelnetConsolePreconfiguredCommandPushButton.setObjectName("uiTelnetConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_9.addWidget(self.uiTelnetConsolePreconfiguredCommandPushButton)
        self.verticalLayout_9.addLayout(self.horizontalLayout_9)
        self.uiSerialConsoleCommandLabel = QtWidgets.QLabel(self.uiTelnetConsoleSettingsGroupBox)
        self.uiSerialConsoleCommandLabel.setObjectName("uiSerialConsoleCommandLabel")
        self.verticalLayout_9.addWidget(self.uiSerialConsoleCommandLabel)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.uiSerialConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSerialConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiSerialConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiSerialConsoleCommandLineEdit.setReadOnly(True)
        self.uiSerialConsoleCommandLineEdit.setObjectName("uiSerialConsoleCommandLineEdit")
        self.horizontalLayout_10.addWidget(self.uiSerialConsoleCommandLineEdit)
        self.uiSerialConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiTelnetConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiSerialConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiSerialConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiSerialConsolePreconfiguredCommandPushButton.setObjectName("uiSerialConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_10.addWidget(self.uiSerialConsolePreconfiguredCommandPushButton)
        self.verticalLayout_9.addLayout(self.horizontalLayout_10)
        self.verticalLayout_3.addWidget(self.uiTelnetConsoleSettingsGroupBox)
        self.uiConsoleMiscGroupBox = QtWidgets.QGroupBox(self.uiConsoleTab)
        self.uiConsoleMiscGroupBox.setObjectName("uiConsoleMiscGroupBox")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.uiConsoleMiscGroupBox)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.uiCloseConsoleWindowsOnDeleteCheckBox = QtWidgets.QCheckBox(self.uiConsoleMiscGroupBox)
        self.uiCloseConsoleWindowsOnDeleteCheckBox.setObjectName("uiCloseConsoleWindowsOnDeleteCheckBox")
        self.gridLayout_7.addWidget(self.uiCloseConsoleWindowsOnDeleteCheckBox, 0, 0, 1, 1)
        self.uiBringConsoleWindowToFrontCheckBox = QtWidgets.QCheckBox(self.uiConsoleMiscGroupBox)
        self.uiBringConsoleWindowToFrontCheckBox.setObjectName("uiBringConsoleWindowToFrontCheckBox")
        self.gridLayout_7.addWidget(self.uiBringConsoleWindowToFrontCheckBox, 1, 0, 1, 1)
        self.uiSlowConsoleAllLabel = QtWidgets.QLabel(self.uiConsoleMiscGroupBox)
        self.uiSlowConsoleAllLabel.setObjectName("uiSlowConsoleAllLabel")
        self.gridLayout_7.addWidget(self.uiSlowConsoleAllLabel, 2, 0, 1, 1)
        self.uiDelayConsoleAllSpinBox = QtWidgets.QSpinBox(self.uiConsoleMiscGroupBox)
        self.uiDelayConsoleAllSpinBox.setMaximum(10000)
        self.uiDelayConsoleAllSpinBox.setProperty("value", 500)
        self.uiDelayConsoleAllSpinBox.setObjectName("uiDelayConsoleAllSpinBox")
        self.gridLayout_7.addWidget(self.uiDelayConsoleAllSpinBox, 3, 0, 1, 1)
        self.verticalLayout_3.addWidget(self.uiConsoleMiscGroupBox)
        spacerItem2 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem2)
        self.uiMiscTabWidget.addTab(self.uiConsoleTab, "")
        self.uiVNCTab = QtWidgets.QWidget()
        self.uiVNCTab.setObjectName("uiVNCTab")
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(self.uiVNCTab)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.uiVNCConsoleSettingsGroupBox = QtWidgets.QGroupBox(self.uiVNCTab)
        self.uiVNCConsoleSettingsGroupBox.setObjectName("uiVNCConsoleSettingsGroupBox")
        self.verticalLayout_8 = QtWidgets.QVBoxLayout(self.uiVNCConsoleSettingsGroupBox)
        self.verticalLayout_8.setObjectName("verticalLayout_8")
        self.uiVNCConsoleCommandLabel = QtWidgets.QLabel(self.uiVNCConsoleSettingsGroupBox)
        self.uiVNCConsoleCommandLabel.setObjectName("uiVNCConsoleCommandLabel")
        self.verticalLayout_8.addWidget(self.uiVNCConsoleCommandLabel)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.uiVNCConsoleCommandLineEdit = QtWidgets.QLineEdit(self.uiVNCConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVNCConsoleCommandLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVNCConsoleCommandLineEdit.setSizePolicy(sizePolicy)
        self.uiVNCConsoleCommandLineEdit.setReadOnly(True)
        self.uiVNCConsoleCommandLineEdit.setObjectName("uiVNCConsoleCommandLineEdit")
        self.horizontalLayout_8.addWidget(self.uiVNCConsoleCommandLineEdit)
        self.uiVNCConsolePreconfiguredCommandPushButton = QtWidgets.QPushButton(self.uiVNCConsoleSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Maximum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVNCConsolePreconfiguredCommandPushButton.sizePolicy().hasHeightForWidth())
        self.uiVNCConsolePreconfiguredCommandPushButton.setSizePolicy(sizePolicy)
        self.uiVNCConsolePreconfiguredCommandPushButton.setObjectName("uiVNCConsolePreconfiguredCommandPushButton")
        self.horizontalLayout_8.addWidget(self.uiVNCConsolePreconfiguredCommandPushButton)
        self.verticalLayout_8.addLayout(self.horizontalLayout_8)
        self.verticalLayout_6.addWidget(self.uiVNCConsoleSettingsGroupBox)
        spacerItem3 = QtWidgets.QSpacerItem(20, 294, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_6.addItem(spacerItem3)
        self.uiMiscTabWidget.addTab(self.uiVNCTab, "")
        self.uiSceneTab = QtWidgets.QWidget()
        self.uiSceneTab.setObjectName("uiSceneTab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.uiSceneTab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.uiSceneWidthLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiSceneWidthLabel.setObjectName("uiSceneWidthLabel")
        self.gridLayout_8.addWidget(self.uiSceneWidthLabel, 0, 0, 1, 1)
        self.uiSceneHeightLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiSceneHeightLabel.setObjectName("uiSceneHeightLabel")
        self.gridLayout_8.addWidget(self.uiSceneHeightLabel, 2, 0, 1, 1)
        self.uiRectangleSelectedItemCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiRectangleSelectedItemCheckBox.setChecked(True)
        self.uiRectangleSelectedItemCheckBox.setObjectName("uiRectangleSelectedItemCheckBox")
        self.gridLayout_8.addWidget(self.uiRectangleSelectedItemCheckBox, 4, 0, 1, 2)
        self.uiDrawLinkStatusPointsCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiDrawLinkStatusPointsCheckBox.setChecked(True)
        self.uiDrawLinkStatusPointsCheckBox.setObjectName("uiDrawLinkStatusPointsCheckBox")
        self.gridLayout_8.addWidget(self.uiDrawLinkStatusPointsCheckBox, 5, 0, 1, 1)
        self.uiLinkManualModeCheckBox = QtWidgets.QCheckBox(self.uiSceneTab)
        self.uiLinkManualModeCheckBox.setChecked(True)
        self.uiLinkManualModeCheckBox.setObjectName("uiLinkManualModeCheckBox")
        self.gridLayout_8.addWidget(self.uiLinkManualModeCheckBox, 6, 0, 1, 2)
        self.uiLabelPreviewLabel = QtWidgets.QLabel(self.uiSceneTab)
        self.uiLabelPreviewLabel.setObjectName("uiLabelPreviewLabel")
        self.gridLayout_8.addWidget(self.uiLabelPreviewLabel, 7, 0, 1, 1)
        self.uiDefaultLabelStylePlainTextEdit = QtWidgets.QPlainTextEdit(self.uiSceneTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiDefaultLabelStylePlainTextEdit.sizePolicy().hasHeightForWidth())
        self.uiDefaultLabelStylePlainTextEdit.setSizePolicy(sizePolicy)
        self.uiDefaultLabelStylePlainTextEdit.setMaximumSize(QtCore.QSize(16777215, 50))
        self.uiDefaultLabelStylePlainTextEdit.setReadOnly(True)
        self.uiDefaultLabelStylePlainTextEdit.setObjectName("uiDefaultLabelStylePlainTextEdit")
        self.gridLayout_8.addWidget(self.uiDefaultLabelStylePlainTextEdit, 8, 0, 1, 2)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiDefaultLabelFontPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultLabelFontPushButton.setObjectName("uiDefaultLabelFontPushButton")
        self.horizontalLayout_5.addWidget(self.uiDefaultLabelFontPushButton)
        self.uiDefaultLabelColorPushButton = QtWidgets.QPushButton(self.uiSceneTab)
        self.uiDefaultLabelColorPushButton.setObjectName("uiDefaultLabelColorPushButton")
        self.horizontalLayout_5.addWidget(self.uiDefaultLabelColorPushButton)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem4)
        self.gridLayout_8.addLayout(self.horizontalLayout_5, 9, 0, 1, 2)
        spacerItem5 = QtWidgets.QSpacerItem(20, 201, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_8.addItem(spacerItem5, 10, 1, 1, 1)
        self.uiSceneHeightSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        self.uiSceneHeightSpinBox.setMinimum(500)
        self.uiSceneHeightSpinBox.setMaximum(1000000)
        self.uiSceneHeightSpinBox.setSingleStep(100)
        self.uiSceneHeightSpinBox.setProperty("value", 1000)
        self.uiSceneHeightSpinBox.setObjectName("uiSceneHeightSpinBox")
        self.gridLayout_8.addWidget(self.uiSceneHeightSpinBox, 3, 0, 1, 2)
        self.uiSceneWidthSpinBox = QtWidgets.QSpinBox(self.uiSceneTab)
        self.uiSceneWidthSpinBox.setMinimum(500)
        self.uiSceneWidthSpinBox.setMaximum(1000000)
        self.uiSceneWidthSpinBox.setSingleStep(100)
        self.uiSceneWidthSpinBox.setProperty("value", 2000)
        self.uiSceneWidthSpinBox.setObjectName("uiSceneWidthSpinBox")
        self.gridLayout_8.addWidget(self.uiSceneWidthSpinBox, 1, 0, 1, 2)
        self.uiMiscTabWidget.addTab(self.uiSceneTab, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.tab)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.uiLaunchNewProjectDialogCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiLaunchNewProjectDialogCheckBox.setChecked(True)
        self.uiLaunchNewProjectDialogCheckBox.setObjectName("uiLaunchNewProjectDialogCheckBox")
        self.verticalLayout_2.addWidget(self.uiLaunchNewProjectDialogCheckBox)
        self.uiAutoScreenshotCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiAutoScreenshotCheckBox.setChecked(True)
        self.uiAutoScreenshotCheckBox.setObjectName("uiAutoScreenshotCheckBox")
        self.verticalLayout_2.addWidget(self.uiAutoScreenshotCheckBox)
        self.uiCheckForUpdateCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiCheckForUpdateCheckBox.setChecked(True)
        self.uiCheckForUpdateCheckBox.setObjectName("uiCheckForUpdateCheckBox")
        self.verticalLayout_2.addWidget(self.uiCheckForUpdateCheckBox)
        self.uiCrashReportCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiCrashReportCheckBox.setChecked(True)
        self.uiCrashReportCheckBox.setObjectName("uiCrashReportCheckBox")
        self.verticalLayout_2.addWidget(self.uiCrashReportCheckBox)
        self.uiStatsCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiStatsCheckBox.setChecked(True)
        self.uiStatsCheckBox.setObjectName("uiStatsCheckBox")
        self.verticalLayout_2.addWidget(self.uiStatsCheckBox)
        self.uiExperimentalFeaturesCheckBox = QtWidgets.QCheckBox(self.tab)
        self.uiExperimentalFeaturesCheckBox.setObjectName("uiExperimentalFeaturesCheckBox")
        self.verticalLayout_2.addWidget(self.uiExperimentalFeaturesCheckBox)
        self.uiSlowStartAllLabel = QtWidgets.QLabel(self.tab)
        self.uiSlowStartAllLabel.setObjectName("uiSlowStartAllLabel")
        self.verticalLayout_2.addWidget(self.uiSlowStartAllLabel)
        self.uiSlowStartAllSpinBox = QtWidgets.QSpinBox(self.tab)
        self.uiSlowStartAllSpinBox.setMinimum(0)
        self.uiSlowStartAllSpinBox.setMaximum(10000)
        self.uiSlowStartAllSpinBox.setProperty("value", 0)
        self.uiSlowStartAllSpinBox.setObjectName("uiSlowStartAllSpinBox")
        self.verticalLayout_2.addWidget(self.uiSlowStartAllSpinBox)
        spacerItem6 = QtWidgets.QSpacerItem(20, 318, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem6)
        self.uiMiscTabWidget.addTab(self.tab, "")
        self.verticalLayout.addWidget(self.uiMiscTabWidget)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        spacerItem7 = QtWidgets.QSpacerItem(324, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem7)
        self.uiRestoreDefaultsPushButton = QtWidgets.QPushButton(GeneralPreferencesPageWidget)
        self.uiRestoreDefaultsPushButton.setObjectName("uiRestoreDefaultsPushButton")
        self.horizontalLayout_6.addWidget(self.uiRestoreDefaultsPushButton)
        self.verticalLayout.addLayout(self.horizontalLayout_6)

        self.retranslateUi(GeneralPreferencesPageWidget)
        self.uiMiscTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(GeneralPreferencesPageWidget)

    def retranslateUi(self, GeneralPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        GeneralPreferencesPageWidget.setWindowTitle(_translate("GeneralPreferencesPageWidget", "General"))
        self.uiLocalPathsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Local paths"))
        self.uiProjectsPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My projects:"))
        self.uiProjectsPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your GNS3 projects are stored"))
        self.uiProjectsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.uiImagesPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My binary images:"))
        self.uiImagesPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your binary images (e.g. IOS) are stored"))
        self.uiImagesPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.label.setText(_translate("GeneralPreferencesPageWidget", "My symbols:"))
        self.uiSymbolsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "Browse..."))
        self.uiConfigsPathLabel.setText(_translate("GeneralPreferencesPageWidget", "My configs:"))
        self.uiConfigsPathLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "Directory where your binary images (e.g. IOS) are stored"))
        self.uiConfigsPathToolButton.setText(_translate("GeneralPreferencesPageWidget", "&Browse..."))
        self.uiStyleGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Style"))
        self.uiConfigurationFileGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Configuration file"))
        self.uiImportConfigurationFilePushButton.setText(_translate("GeneralPreferencesPageWidget", "&Import"))
        self.uiExportConfigurationFilePushButton.setText(_translate("GeneralPreferencesPageWidget", "&Export"))
        self.uiConfigurationFileLabel.setText(_translate("GeneralPreferencesPageWidget", "Unknown location"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiGeneralTab), _translate("GeneralPreferencesPageWidget", "General"))
        self.uiTelnetConsoleSettingsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Console settings"))
        self.uiTelnetConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for Telnet:"))
        self.uiTelnetConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><p>%h = device server </p><p>%p = device port</p><p>%d = device hostname</p></body></html>"))
        self.uiTelnetConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiSerialConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for Serial:"))
        self.uiSerialConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><p>%d = device hostname</p><p>%s = device pipe file</p></body></html>"))
        self.uiSerialConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiConsoleMiscGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Miscellaneous"))
        self.uiCloseConsoleWindowsOnDeleteCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Close any connected console window when deleting a node"))
        self.uiBringConsoleWindowToFrontCheckBox.setToolTip(_translate("GeneralPreferencesPageWidget", "<html>This option will attempt to bring existing opened console window to front, instead of opening a new window.<br>If no existing opened console window exists, it will start a new  console window.</html>"))
        self.uiBringConsoleWindowToFrontCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Bring console window to front (experimental feature)"))
        self.uiSlowConsoleAllLabel.setText(_translate("GeneralPreferencesPageWidget", "Delay between each console launch when consoling to all devices:"))
        self.uiDelayConsoleAllSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " ms"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiConsoleTab), _translate("GeneralPreferencesPageWidget", "Console applications"))
        self.uiVNCConsoleSettingsGroupBox.setTitle(_translate("GeneralPreferencesPageWidget", "Settings for VNC connections"))
        self.uiVNCConsoleCommandLabel.setText(_translate("GeneralPreferencesPageWidget", "Console application command for VNC:"))
        self.uiVNCConsoleCommandLineEdit.setToolTip(_translate("GeneralPreferencesPageWidget", "<html><head/><body><p>Command line replacements:</p><p>%h = device server </p><p>%p = device port</p><p>%d = device hostname</p></body></html>"))
        self.uiVNCConsolePreconfiguredCommandPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Edit"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiVNCTab), _translate("GeneralPreferencesPageWidget", "VNC"))
        self.uiSceneWidthLabel.setText(_translate("GeneralPreferencesPageWidget", "Default width:"))
        self.uiSceneHeightLabel.setText(_translate("GeneralPreferencesPageWidget", "Default height:"))
        self.uiRectangleSelectedItemCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Draw a rectangle when an item is selected"))
        self.uiDrawLinkStatusPointsCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Draw link status points"))
        self.uiLinkManualModeCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Always use manual mode when adding links"))
        self.uiLabelPreviewLabel.setText(_translate("GeneralPreferencesPageWidget", "Default label style:"))
        self.uiDefaultLabelStylePlainTextEdit.setPlainText(_translate("GeneralPreferencesPageWidget", "AaBbYyZz"))
        self.uiDefaultLabelFontPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default font"))
        self.uiDefaultLabelColorPushButton.setText(_translate("GeneralPreferencesPageWidget", "&Select default color"))
        self.uiSceneHeightSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " pixels"))
        self.uiSceneWidthSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " pixels"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.uiSceneTab), _translate("GeneralPreferencesPageWidget", "Topology view"))
        self.uiLaunchNewProjectDialogCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Launch the new project dialog on startup"))
        self.uiAutoScreenshotCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Automatically take a screenshot when saving a project"))
        self.uiCheckForUpdateCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Automatically check for update"))
        self.uiCrashReportCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Send anonymous crash reports"))
        self.uiStatsCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Send anonymous usage statistics"))
        self.uiExperimentalFeaturesCheckBox.setText(_translate("GeneralPreferencesPageWidget", "Enable experimental features (Dangerous, require restart)"))
        self.uiSlowStartAllLabel.setText(_translate("GeneralPreferencesPageWidget", "Delay between each device start when starting all devices:"))
        self.uiSlowStartAllSpinBox.setSuffix(_translate("GeneralPreferencesPageWidget", " seconds"))
        self.uiMiscTabWidget.setTabText(self.uiMiscTabWidget.indexOf(self.tab), _translate("GeneralPreferencesPageWidget", "Miscellaneous"))
        self.uiRestoreDefaultsPushButton.setText(_translate("GeneralPreferencesPageWidget", "Restore defaults"))




# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import tempfile

import logging
log = logging.getLogger(__name__)


class PacketCapture:

    """This class manage packet capture, it's a singleton"""

    def __init__(self):
        self._capture_files = {}

    def startCapture(self, vm, port, file_path):
        """
        Start the packet capture reader on this port

        :param vm: Instance of the virtual machine
        :param port: Instance of port where capture should be executed
        :param file_path: capture file path on server
        """

        if vm.server().isLocal():
            try:
                port.startPacketCapture(vm.name(), file_path)
            except OSError as e:
                vm.error_signal.emit(vm.id(), "Could not start the packet capture reader: {}: {}".format(e, e.filename))
        else:
            (fd, temp_capture_file_path) = tempfile.mkstemp()
            os.close(fd)
            try:
                port.startPacketCapture(vm.name(), temp_capture_file_path)
            except OSError as e:
                vm.error_signal.emit(vm.id(), "Could not start the packet capture reader: {}: {}".format(e, e.filename))
            self._capture_files[port] = temp_capture_file_path

            vm.server().get("/files/stream",
                            None,
                            body={"location": file_path},
                            context={"pcap_file": temp_capture_file_path, "vm": vm},
                            downloadProgressCallback=self._processDownloadPcapProgress,
                            showProgress=False)

        log.info("{} has successfully started capturing packets on {}".format(vm.name(), port.name()))
        vm.updated_signal.emit()

    def _processDownloadPcapProgress(self, content, context={}, **kwargs):

        try:
            with open(context["pcap_file"], 'ab+') as f:
                f.write(content)
        except OSError as e:
            vm = context["vm"]
            vm.error_signal.emit(vm.id(), "Could not write packet capture: {}: {}".format(e, context["pcap_file"]))

    def stopCapture(self, vm, port):
        """
        Stop the packet capture reader on this port

        :param vm: Instance of the virtual machine
        :param port: Instance of port where capture should be executed
        """

        port.stopPacketCapture()
        if port in self._capture_files:
            try:
                os.remove(self._capture_files[port])
            except OSError as e:
                vm.error_signal.emit(vm.id(), "Could not stop packet capture: {}: {}".format(e, self._capture_files[port]))
            self._capture_files[port] = None

        log.info("{} has successfully stopped capturing packets on {}".format(vm.name(), port.name()))
        vm.updated_signal.emit()

    @staticmethod
    def instance():
        """
        Singleton to return only on instance of Servers.

        :returns: instance of Servers
        """

        if not hasattr(PacketCapture, "_instance"):
            PacketCapture._instance = PacketCapture()
        return PacketCapture._instance



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/Users/noplay/code/gns3/gns3-gui/gns3/modules/vmware/ui/vmware_vm_preferences_page.ui'
#
# Created by: PyQt5 UI code generator 5.4.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_VMwareVMPreferencesPageWidget(object):

    def setupUi(self, VMwareVMPreferencesPageWidget):
        VMwareVMPreferencesPageWidget.setObjectName("VMwareVMPreferencesPageWidget")
        VMwareVMPreferencesPageWidget.resize(505, 350)
        VMwareVMPreferencesPageWidget.setAccessibleDescription("")
        self.gridLayout = QtWidgets.QGridLayout(VMwareVMPreferencesPageWidget)
        self.gridLayout.setObjectName("gridLayout")
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.uiNewVMwareVMPushButton = QtWidgets.QPushButton(VMwareVMPreferencesPageWidget)
        self.uiNewVMwareVMPushButton.setObjectName("uiNewVMwareVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiNewVMwareVMPushButton)
        self.uiEditVMwareVMPushButton = QtWidgets.QPushButton(VMwareVMPreferencesPageWidget)
        self.uiEditVMwareVMPushButton.setEnabled(False)
        self.uiEditVMwareVMPushButton.setObjectName("uiEditVMwareVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiEditVMwareVMPushButton)
        self.uiDeleteVMwareVMPushButton = QtWidgets.QPushButton(VMwareVMPreferencesPageWidget)
        self.uiDeleteVMwareVMPushButton.setEnabled(False)
        self.uiDeleteVMwareVMPushButton.setObjectName("uiDeleteVMwareVMPushButton")
        self.horizontalLayout_5.addWidget(self.uiDeleteVMwareVMPushButton)
        self.gridLayout.addLayout(self.horizontalLayout_5, 1, 1, 1, 1)
        self.uiVMwareVMInfoTreeWidget = QtWidgets.QTreeWidget(VMwareVMPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMwareVMInfoTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVMwareVMInfoTreeWidget.setSizePolicy(sizePolicy)
        self.uiVMwareVMInfoTreeWidget.setIndentation(10)
        self.uiVMwareVMInfoTreeWidget.setAllColumnsShowFocus(True)
        self.uiVMwareVMInfoTreeWidget.setObjectName("uiVMwareVMInfoTreeWidget")
        self.uiVMwareVMInfoTreeWidget.header().setVisible(False)
        self.gridLayout.addWidget(self.uiVMwareVMInfoTreeWidget, 0, 1, 1, 1)
        self.uiVMwareVMsTreeWidget = QtWidgets.QTreeWidget(VMwareVMPreferencesPageWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVMwareVMsTreeWidget.sizePolicy().hasHeightForWidth())
        self.uiVMwareVMsTreeWidget.setSizePolicy(sizePolicy)
        self.uiVMwareVMsTreeWidget.setMaximumSize(QtCore.QSize(160, 16777215))
        font = QtGui.QFont()
        font.setPointSize(11)
        font.setBold(True)
        font.setWeight(75)
        self.uiVMwareVMsTreeWidget.setFont(font)
        self.uiVMwareVMsTreeWidget.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.uiVMwareVMsTreeWidget.setIconSize(QtCore.QSize(32, 32))
        self.uiVMwareVMsTreeWidget.setRootIsDecorated(False)
        self.uiVMwareVMsTreeWidget.setObjectName("uiVMwareVMsTreeWidget")
        self.uiVMwareVMsTreeWidget.headerItem().setText(0, "1")
        self.uiVMwareVMsTreeWidget.header().setVisible(False)
        self.gridLayout.addWidget(self.uiVMwareVMsTreeWidget, 0, 0, 2, 1)

        self.retranslateUi(VMwareVMPreferencesPageWidget)
        QtCore.QMetaObject.connectSlotsByName(VMwareVMPreferencesPageWidget)
        VMwareVMPreferencesPageWidget.setTabOrder(self.uiNewVMwareVMPushButton, self.uiDeleteVMwareVMPushButton)

    def retranslateUi(self, VMwareVMPreferencesPageWidget):
        _translate = QtCore.QCoreApplication.translate
        VMwareVMPreferencesPageWidget.setWindowTitle(_translate("VMwareVMPreferencesPageWidget", "VMware VMs"))
        VMwareVMPreferencesPageWidget.setAccessibleName(_translate("VMwareVMPreferencesPageWidget", "VMware VM templates"))
        self.uiNewVMwareVMPushButton.setText(_translate("VMwareVMPreferencesPageWidget", "&New"))
        self.uiEditVMwareVMPushButton.setText(_translate("VMwareVMPreferencesPageWidget", "&Edit"))
        self.uiDeleteVMwareVMPushButton.setText(_translate("VMwareVMPreferencesPageWidget", "&Delete"))
        self.uiVMwareVMInfoTreeWidget.headerItem().setText(0, _translate("VMwareVMPreferencesPageWidget", "1"))
        self.uiVMwareVMInfoTreeWidget.headerItem().setText(1, _translate("VMwareVMPreferencesPageWidget", "2"))



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import time
from contextlib import contextmanager

from .utils import human_filesize
from .qt import QtCore, QtWidgets, Qt, QtNetwork

log = logging.getLogger(__name__)


class Progress(QtCore.QObject):

    """
    Display a progress dialog when something is running
    """

    add_query_signal = QtCore.Signal(str, str, QtNetwork.QNetworkReply)
    remove_query_signal = QtCore.Signal(str)
    progress_signal = QtCore.Signal(str, int, int)

    def __init__(self, parent, min_duration=1000):

        super().__init__(parent)
        self._progress_dialog = None

        # Timer called for refreshing the progress dialog status
        self._rtimer = QtCore.QTimer()
        self._rtimer.timeout.connect(self.update)
        self._rtimer.start(500)

        # When in millisecond we start to show the progress dialog
        self._display_start_time = 0

        self._finished_query_during_display = 0
        self._queries = {}
        # QtCore.Qt.QueuedConnection warranty that we execute the slot
        # in the current thread and not emitter thread.
        # This fix an issue with Qt 5.5
        self.add_query_signal.connect(self._addQuerySlot, QtCore.Qt.QueuedConnection)
        self.remove_query_signal.connect(self._removeQuerySlot, QtCore.Qt.QueuedConnection)
        self.progress_signal.connect(self._progressSlot, QtCore.Qt.QueuedConnection)

        self._minimum_duration = min_duration
        self._cancel_button_text = ""
        self._allow_cancel_query = False
        self._enable = True

    def _addQuerySlot(self, query_id, explanation, response):
        self._queries[query_id] = {"explanation": explanation, "current": 0, "maximum": 0, "response": response}

    def _removeQuerySlot(self, query_id):
        self._finished_query_during_display += 1
        if query_id in self._queries:
            del self._queries[query_id]

    def progress_dialog(self):
        return self._progress_dialog

    def _progressSlot(self, query_id, current, maximum):
        if query_id in self._queries:
            self._queries[query_id]["current"] = current
            self._queries[query_id]["maximum"] = maximum

    def setAllowCancelQuery(self, allow_cancel_query):
        self._allow_cancel_query = allow_cancel_query

    def setCancelButtonText(self, text):
        self._cancel_button_text = text

    def _cancelSlot(self):
        log.debug("User ask for cancel running queries")
        if self._allow_cancel_query:
            log.debug("Cancel running queries")
            for query in self._queries.copy().values():
                query["response"].abort()

    def _rejectSlot(self):
        self._progress_dialog = None
        self._cancelSlot()

    def update(self):
        if len(self._queries) == 0 and (time.time() * 1000) >= self._display_start_time + self._minimum_duration:
            self.hide()
            return
        self.show()

    def show(self):
        if self._progress_dialog is None or self._progress_dialog.wasCanceled():
            progress_dialog = QtWidgets.QProgressDialog("Waiting for server response", None, 0, 0, self.parent())
            progress_dialog.canceled.connect(self._cancelSlot)
            progress_dialog.rejected.connect(self._rejectSlot)
            progress_dialog.setWindowModality(Qt.Qt.ApplicationModal)
            progress_dialog.setWindowTitle("Please wait")
            progress_dialog.setAutoReset(False)
            progress_dialog.setMinimumDuration(self._minimum_duration)

            if len(self._cancel_button_text) > 0:
                progress_dialog.setCancelButtonText(self._cancel_button_text)
            else:
                progress_dialog.setCancelButton(None)

            if len(self._queries) > 0:
                text = list(self._queries.values())[0]["explanation"]
                progress_dialog.setLabelText(text)

            self._progress_dialog = progress_dialog
            self._finished_query_during_display = 0
            self._display_start_time = time.time() * 1000
            self._progress_dialog.show()
        else:
            start_timer = False
            progress_dialog = self._progress_dialog

            if len(self._queries) > 0:
                text = list(self._queries.values())[0]["explanation"]
            else:
                text = None

            # If we have multiple queries running progress show progress of the queries
            # otherwise it's the progress of the current query
            if len(self._queries) + self._finished_query_during_display > 1:
                progress_dialog.setMaximum(len(self._queries) + self._finished_query_during_display)
                progress_dialog.setValue(self._finished_query_during_display)
            elif len(self._queries) == 1:
                query = list(self._queries.values())[0]
                if query["maximum"] == query["current"]:

                    # We animate the bar. In theory Qt should be able to do it but
                    # due to all the manipulation of the dialog he is getting lost
                    bar_speed = 8
                    if progress_dialog.maximum() != bar_speed:
                        progress_dialog.setMaximum(bar_speed)
                        progress_dialog.setValue(0)
                    elif progress_dialog.value() == bar_speed:
                        progress_dialog.setValue(0)
                    else:
                        progress_dialog.setValue(progress_dialog.value() + 1)

                else:
                    progress_dialog.setMaximum(query["maximum"])
                    progress_dialog.setValue(query["current"])

                if text and query["maximum"] > 1000:
                    text += "\n{} / {}".format(human_filesize(query["current"]), human_filesize(query["maximum"]))

            if text:
                progress_dialog.setLabelText(text)

    def hide(self):
        """
        Hide and cancel the progress dialog
        """
        if self._progress_dialog is not None:
            progress_dialog = self._progress_dialog
            self._progress_dialog = None
            progress_dialog.cancel()
            progress_dialog.deleteLater()

    @contextmanager
    def context(self, **kwargs):
        """
        Change the behavior of the progress dialog when in this block
        and restore it at the end of the block.

        :param kwargs: Options to change (possible: min_duration, enable)
        """
        if 'min_duration' in kwargs:
            old_minimum_duration = self._minimum_duration
            self._minimum_duration = kwargs['min_duration']
        if 'enable' in kwargs:
            old_enable = self._enable
            self._enable = kwargs['enable']
        if 'cancel_button_text' in kwargs:
            old_cancel_button_text = self._cancel_button_text
            self._cancel_button_text = kwargs['cancel_button_text']
        if 'allow_cancel_query' in kwargs:
            old_allow_cancel_query = self._allow_cancel_query
            self._allow_cancel_query = kwargs['allow_cancel_query']
        yield
        if 'min_duration' in kwargs:
            self._minimum_duration = old_minimum_duration
        if 'enable' in kwargs:
            self._enable = old_enable
        if 'allow_cancel_query' in kwargs:
            self._allow_cancel_query = old_allow_cancel_query
        if 'cancel_button_text' in kwargs:
            self._cancel_button_text = old_cancel_button_text

    @staticmethod
    def instance(parent=None):
        """
        Singleton to return only one instance of Progress.

        :returns: instance of Progress
        """

        if not hasattr(Progress, "_instance") or Progress._instance is None:
            Progress._instance = Progress(parent)
        return Progress._instance




# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Topology summary view that list all the nodes, their status and connections.
"""

import sip

from .qt import QtGui, QtCore, QtWidgets
from .node import Node
from .topology import Topology
from .items.node_item import NodeItem
from .items.link_item import LinkItem

import logging
log = logging.getLogger(__name__)


class TopologyNodeItem(QtWidgets.QTreeWidgetItem):

    """
    Custom item for the QTreeWidget instance
    (topology summary view).

    :param parent: parent widget
    :param node: Node instance
    """

    def __init__(self, parent, node):

        super().__init__(parent)
        self._node = node
        self._parent = parent

        # we want to know about the node events
        node.started_signal.connect(self._refreshStatusSlot)
        node.stopped_signal.connect(self._refreshStatusSlot)
        node.suspended_signal.connect(self._refreshStatusSlot)
        node.updated_signal.connect(self._refreshNodeSlot)
        node.deleted_signal.connect(self._deletedNodeSlot)

        self._refreshStatusSlot()
        self._refreshNodeSlot()

    def _refreshStatusSlot(self):
        """
        Changes the icon to show the node status (started, stopped etc.)
        """
        if self is None or sip.isdeleted(self):
            return
        self.setText(0, self._node.name())
        if self._node.status() == Node.started:
            self.setIcon(0, QtGui.QIcon(':/icons/led_green.svg'))
        elif self._node.status() == Node.suspended:
            self.setIcon(0, QtGui.QIcon(':/icons/led_yellow.svg'))
        else:
            self.setIcon(0, QtGui.QIcon(':/icons/led_red.svg'))

    def _refreshNodeSlot(self):
        """
        Slot to update the node.
        """

        self.refresh()

    def node(self):
        """
        Returns the node.

        :return: Node instance
        """

        return self._node

    def refresh(self):
        """
        Updates the widget item with the current node name.
        """

        if self._node.name() != self.text(0):
            # refresh all the other item if the node name has changed
            self._parent.refreshAllLinks(source_child=self)
        self.setText(0, self._node.name())
        self.refreshLinks()
        self._parent.invisibleRootItem().sortChildren(0, QtCore.Qt.AscendingOrder)

    def refreshLinks(self):
        """
        List all the connections as children.
        """

        ports = self._node.ports()
        self.takeChildren()

        capturing = False
        for port in ports:
            if not port.isFree():
                item = QtWidgets.QTreeWidgetItem()
                item.setText(0, "{} {}".format(port.shortName(), port.description(short=True)))
                item.setData(0, QtCore.Qt.UserRole, port)
                if port.capturing():
                    item.setIcon(0, QtGui.QIcon(':/icons/inspect.svg'))
                    capturing = True
                self.addChild(item)

        if self._parent.show_only_devices_with_capture and capturing is False:
            self.setHidden(True)
        else:
            self.setHidden(False)

        self.sortChildren(0, QtCore.Qt.AscendingOrder)

    def _deletedNodeSlot(self):
        """
        Removes the node from the view.
        """

        tree = self.treeWidget()
        tree.takeTopLevelItem(tree.indexOfTopLevelItem(self))


class TopologySummaryView(QtWidgets.QTreeWidget):

    """
    Topology summary view implementation.

    :param parent: parent widget
    """

    def __init__(self, parent):

        super().__init__(parent)
        self._topology = Topology.instance()
        self.itemSelectionChanged.connect(self._itemSelectionChangedSlot)
        self.show_only_devices_with_capture = False
        self.setExpandsOnDoubleClick(False)
        self.itemDoubleClicked.connect(self._itemDoubleClickedSlot)

    def addNode(self, node):
        """
        Adds a node to the summary view.

        :param node: Node instance
        """

        # we want to have this node listed only when completely created.
        node.created_signal.connect(self._createdNodeSlot)

    def clear(self):
        """
        Clears all the topology summary.
        """

        QtWidgets.QTreeWidget.clear(self)

    def refreshAllLinks(self, source_child=None):
        """
        Refreshes all links for all items.
        """

        root = self.invisibleRootItem()
        for index in range(0, root.childCount()):
            child = root.child(index)
            if source_child and source_child == child:
                continue
            child.refreshLinks()

    def _createdNodeSlot(self, node_id):
        """
        Received events for node creation.

        :param node_id: node identifier
        """

        if not node_id:
            log.error("node ID is null")
            return

        node = self._topology.getNode(node_id)
        if not node:
            log.error("could not find node with ID {}".format(node_id))
            return

        TopologyNodeItem(self, node)

    def _itemSelectionChangedSlot(self):
        """
        Slot called when an item is selected in the TreeWidget.
        """

        current_item = self.currentItem()
        if current_item:
            from .main_window import MainWindow
            view = MainWindow.instance().uiGraphicsView
            for item in view.scene().items():
                if isinstance(item, NodeItem):
                    item.setSelected(False)
                    if isinstance(current_item, TopologyNodeItem) and item.node().id() == current_item.node().id():
                        item.setSelected(True)
                elif isinstance(item, LinkItem):
                    item.setHovered(False)
                    if not isinstance(current_item, TopologyNodeItem):
                        port = current_item.data(0, QtCore.Qt.UserRole)
                        if item.sourcePort() == port or item.destinationPort() == port:
                            item.setHovered(True)

    def _itemDoubleClickedSlot(self, current_item):
        """
        When user double click on an element we center the topology on it
        """
        if current_item != 0:
            from .main_window import MainWindow
            view = MainWindow.instance().uiGraphicsView
            for item in view.scene().items():
                if isinstance(item, NodeItem):
                    if isinstance(current_item, TopologyNodeItem) and item.node().id() == current_item.node().id():
                        view.centerOn(item)
                elif isinstance(item, LinkItem):
                    if not isinstance(current_item, TopologyNodeItem):
                        port = current_item.data(0, QtCore.Qt.UserRole)
                        if item.sourcePort() == port or item.destinationPort() == port:
                            view.centerOn(item)

    def mousePressEvent(self, event):
        """
        Handles all mouse press events.

        :param event: QMouseEvent instance
        """

        if event.button() == QtCore.Qt.RightButton:
            self._showContextualMenu()
        else:
            super().mousePressEvent(event)


    def _showContextualMenu(self):
        """
        Contextual menu to expand and collapse the tree.
        """

        menu = QtWidgets.QMenu()
        expand_all = QtWidgets.QAction("Expand all", menu)
        expand_all.setIcon(QtGui.QIcon(":/icons/plus.svg"))
        expand_all.triggered.connect(self._expandAllSlot)
        menu.addAction(expand_all)

        collapse_all = QtWidgets.QAction("Collapse all", menu)
        collapse_all.setIcon(QtGui.QIcon(":/icons/minus.svg"))
        collapse_all.triggered.connect(self._collapseAllSlot)
        menu.addAction(collapse_all)

        if self.show_only_devices_with_capture is False:
            devices_with_capture = QtWidgets.QAction("Show devices with capture(s)", menu)
            devices_with_capture.setIcon(QtGui.QIcon(":/icons/inspect.svg"))
            devices_with_capture.triggered.connect(self._devicesWithCaptureSlot)
            menu.addAction(devices_with_capture)
        else:
            show_all_devices = QtWidgets.QAction("Show all devices", menu)
            # show_all_devices.setIcon(QtGui.QIcon(":/icons/inspect.svg"))
            show_all_devices.triggered.connect(self._showAllDevicesSlot)
            menu.addAction(show_all_devices)

        stop_all_captures = QtWidgets.QAction("Stop all captures", menu)
        stop_all_captures.setIcon(QtGui.QIcon(":/icons/capture-stop.svg"))
        stop_all_captures.triggered.connect(self._stopAllCapturesSlot)
        menu.addAction(stop_all_captures)

        current_item = self.currentItem()
        from .main_window import MainWindow
        view = MainWindow.instance().uiGraphicsView
        if current_item and not current_item.isHidden():
            menu.addSeparator()
            if isinstance(current_item, TopologyNodeItem):
                view.populateDeviceContextualMenu(menu)
            else:
                port = current_item.data(0, QtCore.Qt.UserRole)
                for item in view.scene().items():
                    if isinstance(item, LinkItem) and (item.sourcePort() == port or item.destinationPort() == port):
                        item.populateLinkContextualMenu(menu)
                        break

        menu.exec_(QtGui.QCursor.pos())

    def _expandAllSlot(self):
        """
        Expands all items.
        """

        self.expandAll()

    def _collapseAllSlot(self):
        """
        Collapses all items.
        """

        self.collapseAll()

    def _devicesWithCaptureSlot(self):
        """
        Show only devices with captures.
        """

        self.show_only_devices_with_capture = True
        self.refreshAllLinks()

    def _showAllDevicesSlot(self):
        """
        Show all devices items.
        """

        self.show_only_devices_with_capture = False
        self.refreshAllLinks()

    def _stopAllCapturesSlot(self):
        """
        Stop all packet captures.
        """

        for node in self._topology.nodes():
            if hasattr(node, "stopPacketCapture"):
                for port in node.ports():
                    if port.capturing():
                        node.stopPacketCapture(port)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2013 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
from setuptools import setup, find_packages
from setuptools.command.test import test as TestCommand

# we only support Python 3 version >= 3.4
if sys.version_info < (3, 4):
    raise SystemExit("Python 3.4 or higher is required")


class Tox(TestCommand):

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        # import here, cause outside the eggs aren't loaded
        import tox
        errcode = tox.cmdline(self.test_args)
        sys.exit(errcode)

setup(
    name="gns3-gui",
    version=__import__("gns3").__version__,
    url="http://github.com/GNS3/gns3-gui",
    license="GNU General Public License v3 (GPLv3)",
    tests_require=["tox"],
    cmdclass={"test": Tox},
    author="Jeremy Grossmann",
    author_email="package-maintainer@gns3.net",
    description="GNS3 graphical interface for the GNS3 server.",
    long_description=open("README.rst", "r").read(),
    install_requires=[
        "jsonschema>=2.4.0",
        "gns3-net-converter>=1.3.0",
        "raven>=5.2.0",
        "psutil>=2.2.1",
    ],
    entry_points={
        "gui_scripts": [
            "gns3 = gns3.main:main",
            "gns3-iouvm-converter = gns3.iouvm_converter:main"
        ]
    },
    packages=find_packages(".", exclude=["docs", "tests"]),
    include_package_data=True,
    package_data={"gns3": ["configs/*.txt", "schemas/*.json"]},
    platforms="any",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: X11 Applications :: Qt",
        "Intended Audience :: Information Technology",
        "Topic :: System :: Networking",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        'Natural Language :: English',
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: Implementation :: CPython",
    ],
)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil

from gns3.servers import Servers

import logging
log = logging.getLogger(__name__)


def get_default_base_config(base_config_template_path):
    """
    Copy the default base config template to settings directory (if not already present) and returns the path.

    :param base_config_template_path: path to the base config template

    :return: path to the base config
    """

    servers = Servers.instance()
    config_dir = servers.localServerSettings()["configs_path"]
    if base_config_template_path:
        try:
            os.makedirs(config_dir, exist_ok=True)
        except OSError as e:
            log.error("could not create the base configs directory {}: {}".format(config_dir, e))
            return ""
        try:
            base_config_path = os.path.join(config_dir, os.path.basename(base_config_template_path))
            if not os.path.isfile(base_config_path):
                shutil.copyfile(base_config_template_path, base_config_path)
            return os.path.normpath(base_config_path)
        except OSError as e:
            log.error("could not copy {} to {}: {}".format(base_config_template_path, base_config_path, e))
    return ""



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file '/home/grossmj/PycharmProjects/gns3-gui/gns3/modules/builtin/ui/cloud_configuration_page.ui'
#
# Created: Sat Mar 19 22:31:09 2016
#      by: PyQt5 UI code generator 5.2.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_cloudConfigPageWidget(object):
    def setupUi(self, cloudConfigPageWidget):
        cloudConfigPageWidget.setObjectName("cloudConfigPageWidget")
        cloudConfigPageWidget.resize(653, 478)
        self.vboxlayout = QtWidgets.QVBoxLayout(cloudConfigPageWidget)
        self.vboxlayout.setObjectName("vboxlayout")
        self.uiNIOsTabWidget = QtWidgets.QTabWidget(cloudConfigPageWidget)
        self.uiNIOsTabWidget.setObjectName("uiNIOsTabWidget")
        self.NIOEthernetTab = QtWidgets.QWidget()
        self.NIOEthernetTab.setObjectName("NIOEthernetTab")
        self.vboxlayout1 = QtWidgets.QVBoxLayout(self.NIOEthernetTab)
        self.vboxlayout1.setObjectName("vboxlayout1")
        self.uiGenericEthernetGroupBox = QtWidgets.QGroupBox(self.NIOEthernetTab)
        self.uiGenericEthernetGroupBox.setObjectName("uiGenericEthernetGroupBox")
        self.gridlayout = QtWidgets.QGridLayout(self.uiGenericEthernetGroupBox)
        self.gridlayout.setObjectName("gridlayout")
        self.uiGenericEthernetComboBox = QtWidgets.QComboBox(self.uiGenericEthernetGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiGenericEthernetComboBox.sizePolicy().hasHeightForWidth())
        self.uiGenericEthernetComboBox.setSizePolicy(sizePolicy)
        self.uiGenericEthernetComboBox.setSizeAdjustPolicy(QtWidgets.QComboBox.AdjustToContents)
        self.uiGenericEthernetComboBox.setObjectName("uiGenericEthernetComboBox")
        self.gridlayout.addWidget(self.uiGenericEthernetComboBox, 0, 0, 1, 3)
        self.uiGenericEthernetLineEdit = QtWidgets.QLineEdit(self.uiGenericEthernetGroupBox)
        self.uiGenericEthernetLineEdit.setObjectName("uiGenericEthernetLineEdit")
        self.gridlayout.addWidget(self.uiGenericEthernetLineEdit, 1, 0, 1, 1)
        self.uiAddGenericEthernetPushButton = QtWidgets.QPushButton(self.uiGenericEthernetGroupBox)
        self.uiAddGenericEthernetPushButton.setObjectName("uiAddGenericEthernetPushButton")
        self.gridlayout.addWidget(self.uiAddGenericEthernetPushButton, 1, 1, 1, 1)
        self.uiDeleteGenericEthernetPushButton = QtWidgets.QPushButton(self.uiGenericEthernetGroupBox)
        self.uiDeleteGenericEthernetPushButton.setEnabled(False)
        self.uiDeleteGenericEthernetPushButton.setObjectName("uiDeleteGenericEthernetPushButton")
        self.gridlayout.addWidget(self.uiDeleteGenericEthernetPushButton, 1, 2, 1, 1)
        self.uiGenericEthernetListWidget = QtWidgets.QListWidget(self.uiGenericEthernetGroupBox)
        self.uiGenericEthernetListWidget.setObjectName("uiGenericEthernetListWidget")
        self.gridlayout.addWidget(self.uiGenericEthernetListWidget, 2, 0, 1, 3)
        self.vboxlayout1.addWidget(self.uiGenericEthernetGroupBox)
        self.uiLinuxEthernetGroupBox = QtWidgets.QGroupBox(self.NIOEthernetTab)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLinuxEthernetGroupBox.sizePolicy().hasHeightForWidth())
        self.uiLinuxEthernetGroupBox.setSizePolicy(sizePolicy)
        self.uiLinuxEthernetGroupBox.setObjectName("uiLinuxEthernetGroupBox")
        self.gridlayout1 = QtWidgets.QGridLayout(self.uiLinuxEthernetGroupBox)
        self.gridlayout1.setObjectName("gridlayout1")
        self.uiLinuxEthernetComboBox = QtWidgets.QComboBox(self.uiLinuxEthernetGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLinuxEthernetComboBox.sizePolicy().hasHeightForWidth())
        self.uiLinuxEthernetComboBox.setSizePolicy(sizePolicy)
        self.uiLinuxEthernetComboBox.setObjectName("uiLinuxEthernetComboBox")
        self.gridlayout1.addWidget(self.uiLinuxEthernetComboBox, 0, 0, 1, 3)
        self.uiLinuxEthernetLineEdit = QtWidgets.QLineEdit(self.uiLinuxEthernetGroupBox)
        self.uiLinuxEthernetLineEdit.setObjectName("uiLinuxEthernetLineEdit")
        self.gridlayout1.addWidget(self.uiLinuxEthernetLineEdit, 1, 0, 1, 1)
        self.uiAddLinuxEthernetPushButton = QtWidgets.QPushButton(self.uiLinuxEthernetGroupBox)
        self.uiAddLinuxEthernetPushButton.setObjectName("uiAddLinuxEthernetPushButton")
        self.gridlayout1.addWidget(self.uiAddLinuxEthernetPushButton, 1, 1, 1, 1)
        self.uiDeleteLinuxEthernetPushButton = QtWidgets.QPushButton(self.uiLinuxEthernetGroupBox)
        self.uiDeleteLinuxEthernetPushButton.setEnabled(False)
        self.uiDeleteLinuxEthernetPushButton.setObjectName("uiDeleteLinuxEthernetPushButton")
        self.gridlayout1.addWidget(self.uiDeleteLinuxEthernetPushButton, 1, 2, 1, 1)
        self.uiLinuxEthernetListWidget = QtWidgets.QListWidget(self.uiLinuxEthernetGroupBox)
        self.uiLinuxEthernetListWidget.setObjectName("uiLinuxEthernetListWidget")
        self.gridlayout1.addWidget(self.uiLinuxEthernetListWidget, 2, 0, 1, 3)
        self.vboxlayout1.addWidget(self.uiLinuxEthernetGroupBox)
        spacerItem = QtWidgets.QSpacerItem(21, 16, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.vboxlayout1.addItem(spacerItem)
        self.uiNIOsTabWidget.addTab(self.NIOEthernetTab, "")
        self.NIONATTab = QtWidgets.QWidget()
        self.NIONATTab.setObjectName("NIONATTab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.NIONATTab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.uiNIONATSettingsGroupBox = QtWidgets.QGroupBox(self.NIONATTab)
        self.uiNIONATSettingsGroupBox.setObjectName("uiNIONATSettingsGroupBox")
        self._2 = QtWidgets.QGridLayout(self.uiNIONATSettingsGroupBox)
        self._2.setObjectName("_2")
        self.uiNIONATIdentifierLabel = QtWidgets.QLabel(self.uiNIONATSettingsGroupBox)
        self.uiNIONATIdentifierLabel.setObjectName("uiNIONATIdentifierLabel")
        self._2.addWidget(self.uiNIONATIdentifierLabel, 0, 0, 1, 1)
        self.uiNIONATIdentiferLineEdit = QtWidgets.QLineEdit(self.uiNIONATSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIONATIdentiferLineEdit.sizePolicy().hasHeightForWidth())
        self.uiNIONATIdentiferLineEdit.setSizePolicy(sizePolicy)
        self.uiNIONATIdentiferLineEdit.setObjectName("uiNIONATIdentiferLineEdit")
        self._2.addWidget(self.uiNIONATIdentiferLineEdit, 1, 0, 1, 1)
        self.gridLayout_2.addWidget(self.uiNIONATSettingsGroupBox, 0, 0, 1, 2)
        self.uiNIONATListGroupBox = QtWidgets.QGroupBox(self.NIONATTab)
        self.uiNIONATListGroupBox.setObjectName("uiNIONATListGroupBox")
        self._3 = QtWidgets.QVBoxLayout(self.uiNIONATListGroupBox)
        self._3.setObjectName("_3")
        self.uiNIONATListWidget = QtWidgets.QListWidget(self.uiNIONATListGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIONATListWidget.sizePolicy().hasHeightForWidth())
        self.uiNIONATListWidget.setSizePolicy(sizePolicy)
        self.uiNIONATListWidget.setObjectName("uiNIONATListWidget")
        self._3.addWidget(self.uiNIONATListWidget)
        self.gridLayout_2.addWidget(self.uiNIONATListGroupBox, 0, 2, 3, 1)
        self.uiAddNIONATPushButton = QtWidgets.QPushButton(self.NIONATTab)
        self.uiAddNIONATPushButton.setObjectName("uiAddNIONATPushButton")
        self.gridLayout_2.addWidget(self.uiAddNIONATPushButton, 1, 0, 1, 1)
        self.uiDeleteNIONATPushButton = QtWidgets.QPushButton(self.NIONATTab)
        self.uiDeleteNIONATPushButton.setEnabled(False)
        self.uiDeleteNIONATPushButton.setObjectName("uiDeleteNIONATPushButton")
        self.gridLayout_2.addWidget(self.uiDeleteNIONATPushButton, 1, 1, 1, 1)
        spacerItem1 = QtWidgets.QSpacerItem(20, 294, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem1, 2, 0, 2, 1)
        spacerItem2 = QtWidgets.QSpacerItem(20, 194, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout_2.addItem(spacerItem2, 3, 2, 1, 1)
        self.uiNIOsTabWidget.addTab(self.NIONATTab, "")
        self.NIOUDPTab = QtWidgets.QWidget()
        self.NIOUDPTab.setObjectName("NIOUDPTab")
        self.gridlayout2 = QtWidgets.QGridLayout(self.NIOUDPTab)
        self.gridlayout2.setObjectName("gridlayout2")
        self.uiNIOUDPSettingsGroupBox = QtWidgets.QGroupBox(self.NIOUDPTab)
        self.uiNIOUDPSettingsGroupBox.setObjectName("uiNIOUDPSettingsGroupBox")
        self.gridlayout3 = QtWidgets.QGridLayout(self.uiNIOUDPSettingsGroupBox)
        self.gridlayout3.setObjectName("gridlayout3")
        self.uiLocalPortLabel = QtWidgets.QLabel(self.uiNIOUDPSettingsGroupBox)
        self.uiLocalPortLabel.setObjectName("uiLocalPortLabel")
        self.gridlayout3.addWidget(self.uiLocalPortLabel, 0, 0, 1, 1)
        self.uiLocalPortSpinBox = QtWidgets.QSpinBox(self.uiNIOUDPSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalPortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiLocalPortSpinBox.setSizePolicy(sizePolicy)
        self.uiLocalPortSpinBox.setMaximum(65535)
        self.uiLocalPortSpinBox.setProperty("value", 30000)
        self.uiLocalPortSpinBox.setObjectName("uiLocalPortSpinBox")
        self.gridlayout3.addWidget(self.uiLocalPortSpinBox, 0, 1, 1, 1)
        self.uiRemoteHostLabel = QtWidgets.QLabel(self.uiNIOUDPSettingsGroupBox)
        self.uiRemoteHostLabel.setObjectName("uiRemoteHostLabel")
        self.gridlayout3.addWidget(self.uiRemoteHostLabel, 1, 0, 1, 1)
        self.uiRemoteHostLineEdit = QtWidgets.QLineEdit(self.uiNIOUDPSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteHostLineEdit.sizePolicy().hasHeightForWidth())
        self.uiRemoteHostLineEdit.setSizePolicy(sizePolicy)
        self.uiRemoteHostLineEdit.setMinimumSize(QtCore.QSize(80, 0))
        self.uiRemoteHostLineEdit.setObjectName("uiRemoteHostLineEdit")
        self.gridlayout3.addWidget(self.uiRemoteHostLineEdit, 1, 1, 1, 1)
        self.uiRemotePortLabel = QtWidgets.QLabel(self.uiNIOUDPSettingsGroupBox)
        self.uiRemotePortLabel.setObjectName("uiRemotePortLabel")
        self.gridlayout3.addWidget(self.uiRemotePortLabel, 2, 0, 1, 1)
        self.uiRemotePortSpinBox = QtWidgets.QSpinBox(self.uiNIOUDPSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemotePortSpinBox.sizePolicy().hasHeightForWidth())
        self.uiRemotePortSpinBox.setSizePolicy(sizePolicy)
        self.uiRemotePortSpinBox.setMaximum(65535)
        self.uiRemotePortSpinBox.setProperty("value", 20000)
        self.uiRemotePortSpinBox.setObjectName("uiRemotePortSpinBox")
        self.gridlayout3.addWidget(self.uiRemotePortSpinBox, 2, 1, 1, 1)
        self.gridlayout2.addWidget(self.uiNIOUDPSettingsGroupBox, 0, 0, 1, 2)
        self.uiNIOUDPListGroupBox = QtWidgets.QGroupBox(self.NIOUDPTab)
        self.uiNIOUDPListGroupBox.setObjectName("uiNIOUDPListGroupBox")
        self.vboxlayout2 = QtWidgets.QVBoxLayout(self.uiNIOUDPListGroupBox)
        self.vboxlayout2.setObjectName("vboxlayout2")
        self.uiNIOUDPListWidget = QtWidgets.QListWidget(self.uiNIOUDPListGroupBox)
        self.uiNIOUDPListWidget.setObjectName("uiNIOUDPListWidget")
        self.vboxlayout2.addWidget(self.uiNIOUDPListWidget)
        self.gridlayout2.addWidget(self.uiNIOUDPListGroupBox, 0, 2, 2, 1)
        self.uiAddNIOUDPPushButton = QtWidgets.QPushButton(self.NIOUDPTab)
        self.uiAddNIOUDPPushButton.setObjectName("uiAddNIOUDPPushButton")
        self.gridlayout2.addWidget(self.uiAddNIOUDPPushButton, 1, 0, 1, 1)
        self.uiDeleteNIOUDPPushButton = QtWidgets.QPushButton(self.NIOUDPTab)
        self.uiDeleteNIOUDPPushButton.setEnabled(False)
        self.uiDeleteNIOUDPPushButton.setObjectName("uiDeleteNIOUDPPushButton")
        self.gridlayout2.addWidget(self.uiDeleteNIOUDPPushButton, 1, 1, 1, 1)
        spacerItem3 = QtWidgets.QSpacerItem(20, 211, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridlayout2.addItem(spacerItem3, 2, 1, 1, 1)
        self.uiNIOsTabWidget.addTab(self.NIOUDPTab, "")
        self.NIOTAPTab = QtWidgets.QWidget()
        self.NIOTAPTab.setObjectName("NIOTAPTab")
        self.vboxlayout3 = QtWidgets.QVBoxLayout(self.NIOTAPTab)
        self.vboxlayout3.setObjectName("vboxlayout3")
        self.uiNIOTAPGroupBox = QtWidgets.QGroupBox(self.NIOTAPTab)
        self.uiNIOTAPGroupBox.setObjectName("uiNIOTAPGroupBox")
        self.gridlayout4 = QtWidgets.QGridLayout(self.uiNIOTAPGroupBox)
        self.gridlayout4.setObjectName("gridlayout4")
        self.uiNIOTAPLineEdit = QtWidgets.QLineEdit(self.uiNIOTAPGroupBox)
        self.uiNIOTAPLineEdit.setObjectName("uiNIOTAPLineEdit")
        self.gridlayout4.addWidget(self.uiNIOTAPLineEdit, 0, 0, 1, 1)
        self.uiAddNIOTAPPushButton = QtWidgets.QPushButton(self.uiNIOTAPGroupBox)
        self.uiAddNIOTAPPushButton.setObjectName("uiAddNIOTAPPushButton")
        self.gridlayout4.addWidget(self.uiAddNIOTAPPushButton, 0, 1, 1, 1)
        self.uiDeleteNIOTAPPushButton = QtWidgets.QPushButton(self.uiNIOTAPGroupBox)
        self.uiDeleteNIOTAPPushButton.setEnabled(False)
        self.uiDeleteNIOTAPPushButton.setObjectName("uiDeleteNIOTAPPushButton")
        self.gridlayout4.addWidget(self.uiDeleteNIOTAPPushButton, 0, 2, 1, 1)
        self.uiNIOTAPListWidget = QtWidgets.QListWidget(self.uiNIOTAPGroupBox)
        self.uiNIOTAPListWidget.setObjectName("uiNIOTAPListWidget")
        self.gridlayout4.addWidget(self.uiNIOTAPListWidget, 1, 0, 1, 3)
        self.vboxlayout3.addWidget(self.uiNIOTAPGroupBox)
        spacerItem4 = QtWidgets.QSpacerItem(20, 191, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.vboxlayout3.addItem(spacerItem4)
        self.uiNIOsTabWidget.addTab(self.NIOTAPTab, "")
        self.NIOUnixTab = QtWidgets.QWidget()
        self.NIOUnixTab.setObjectName("NIOUnixTab")
        self.gridlayout5 = QtWidgets.QGridLayout(self.NIOUnixTab)
        self.gridlayout5.setObjectName("gridlayout5")
        self.uiNIOUNIXSettingsGroupBox = QtWidgets.QGroupBox(self.NIOUnixTab)
        self.uiNIOUNIXSettingsGroupBox.setObjectName("uiNIOUNIXSettingsGroupBox")
        self.gridlayout6 = QtWidgets.QGridLayout(self.uiNIOUNIXSettingsGroupBox)
        self.gridlayout6.setObjectName("gridlayout6")
        self.gridlayout7 = QtWidgets.QGridLayout()
        self.gridlayout7.setObjectName("gridlayout7")
        self.uiLocalFileLabel = QtWidgets.QLabel(self.uiNIOUNIXSettingsGroupBox)
        self.uiLocalFileLabel.setObjectName("uiLocalFileLabel")
        self.gridlayout7.addWidget(self.uiLocalFileLabel, 0, 0, 1, 1)
        self.uiLocalFileLineEdit = QtWidgets.QLineEdit(self.uiNIOUNIXSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiLocalFileLineEdit.sizePolicy().hasHeightForWidth())
        self.uiLocalFileLineEdit.setSizePolicy(sizePolicy)
        self.uiLocalFileLineEdit.setObjectName("uiLocalFileLineEdit")
        self.gridlayout7.addWidget(self.uiLocalFileLineEdit, 1, 0, 1, 1)
        self.gridlayout6.addLayout(self.gridlayout7, 0, 0, 1, 1)
        self.gridlayout8 = QtWidgets.QGridLayout()
        self.gridlayout8.setObjectName("gridlayout8")
        self.uiRemoteFileLabel = QtWidgets.QLabel(self.uiNIOUNIXSettingsGroupBox)
        self.uiRemoteFileLabel.setObjectName("uiRemoteFileLabel")
        self.gridlayout8.addWidget(self.uiRemoteFileLabel, 0, 0, 1, 1)
        self.uiRemoteFileLineEdit = QtWidgets.QLineEdit(self.uiNIOUNIXSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiRemoteFileLineEdit.sizePolicy().hasHeightForWidth())
        self.uiRemoteFileLineEdit.setSizePolicy(sizePolicy)
        self.uiRemoteFileLineEdit.setObjectName("uiRemoteFileLineEdit")
        self.gridlayout8.addWidget(self.uiRemoteFileLineEdit, 1, 0, 1, 1)
        self.gridlayout6.addLayout(self.gridlayout8, 1, 0, 1, 1)
        self.gridlayout5.addWidget(self.uiNIOUNIXSettingsGroupBox, 0, 0, 1, 2)
        self.uiNIOUNIXListGroupBox = QtWidgets.QGroupBox(self.NIOUnixTab)
        self.uiNIOUNIXListGroupBox.setObjectName("uiNIOUNIXListGroupBox")
        self.vboxlayout4 = QtWidgets.QVBoxLayout(self.uiNIOUNIXListGroupBox)
        self.vboxlayout4.setObjectName("vboxlayout4")
        self.uiNIOUNIXListWidget = QtWidgets.QListWidget(self.uiNIOUNIXListGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIOUNIXListWidget.sizePolicy().hasHeightForWidth())
        self.uiNIOUNIXListWidget.setSizePolicy(sizePolicy)
        self.uiNIOUNIXListWidget.setObjectName("uiNIOUNIXListWidget")
        self.vboxlayout4.addWidget(self.uiNIOUNIXListWidget)
        self.gridlayout5.addWidget(self.uiNIOUNIXListGroupBox, 0, 2, 3, 1)
        self.uiAddNIOUNIXPushButton = QtWidgets.QPushButton(self.NIOUnixTab)
        self.uiAddNIOUNIXPushButton.setObjectName("uiAddNIOUNIXPushButton")
        self.gridlayout5.addWidget(self.uiAddNIOUNIXPushButton, 1, 0, 1, 1)
        self.uiDeleteNIOUNIXPushButton = QtWidgets.QPushButton(self.NIOUnixTab)
        self.uiDeleteNIOUNIXPushButton.setEnabled(False)
        self.uiDeleteNIOUNIXPushButton.setObjectName("uiDeleteNIOUNIXPushButton")
        self.gridlayout5.addWidget(self.uiDeleteNIOUNIXPushButton, 1, 1, 1, 1)
        spacerItem5 = QtWidgets.QSpacerItem(160, 190, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridlayout5.addItem(spacerItem5, 2, 0, 2, 2)
        spacerItem6 = QtWidgets.QSpacerItem(196, 132, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridlayout5.addItem(spacerItem6, 3, 2, 1, 1)
        self.uiNIOsTabWidget.addTab(self.NIOUnixTab, "")
        self.NIOVDETab = QtWidgets.QWidget()
        self.NIOVDETab.setObjectName("NIOVDETab")
        self.gridlayout9 = QtWidgets.QGridLayout(self.NIOVDETab)
        self.gridlayout9.setObjectName("gridlayout9")
        self.uiNIOVDESettingsGroupBox = QtWidgets.QGroupBox(self.NIOVDETab)
        self.uiNIOVDESettingsGroupBox.setObjectName("uiNIOVDESettingsGroupBox")
        self.gridlayout10 = QtWidgets.QGridLayout(self.uiNIOVDESettingsGroupBox)
        self.gridlayout10.setObjectName("gridlayout10")
        self.gridlayout11 = QtWidgets.QGridLayout()
        self.gridlayout11.setObjectName("gridlayout11")
        self.uiVDEControlFileLabel = QtWidgets.QLabel(self.uiNIOVDESettingsGroupBox)
        self.uiVDEControlFileLabel.setObjectName("uiVDEControlFileLabel")
        self.gridlayout11.addWidget(self.uiVDEControlFileLabel, 0, 0, 1, 1)
        self.uiVDEControlFileLineEdit = QtWidgets.QLineEdit(self.uiNIOVDESettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVDEControlFileLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVDEControlFileLineEdit.setSizePolicy(sizePolicy)
        self.uiVDEControlFileLineEdit.setObjectName("uiVDEControlFileLineEdit")
        self.gridlayout11.addWidget(self.uiVDEControlFileLineEdit, 1, 0, 1, 1)
        self.gridlayout10.addLayout(self.gridlayout11, 0, 0, 1, 1)
        self.gridlayout12 = QtWidgets.QGridLayout()
        self.gridlayout12.setObjectName("gridlayout12")
        self.uiVDELocalFileLabel = QtWidgets.QLabel(self.uiNIOVDESettingsGroupBox)
        self.uiVDELocalFileLabel.setObjectName("uiVDELocalFileLabel")
        self.gridlayout12.addWidget(self.uiVDELocalFileLabel, 0, 0, 1, 1)
        self.uiVDELocalFileLineEdit = QtWidgets.QLineEdit(self.uiNIOVDESettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiVDELocalFileLineEdit.sizePolicy().hasHeightForWidth())
        self.uiVDELocalFileLineEdit.setSizePolicy(sizePolicy)
        self.uiVDELocalFileLineEdit.setObjectName("uiVDELocalFileLineEdit")
        self.gridlayout12.addWidget(self.uiVDELocalFileLineEdit, 1, 0, 1, 1)
        self.gridlayout10.addLayout(self.gridlayout12, 1, 0, 1, 1)
        self.gridlayout9.addWidget(self.uiNIOVDESettingsGroupBox, 0, 0, 1, 2)
        self.uiNIOVDEListGroupBox = QtWidgets.QGroupBox(self.NIOVDETab)
        self.uiNIOVDEListGroupBox.setObjectName("uiNIOVDEListGroupBox")
        self.vboxlayout5 = QtWidgets.QVBoxLayout(self.uiNIOVDEListGroupBox)
        self.vboxlayout5.setObjectName("vboxlayout5")
        self.uiNIOVDEListWidget = QtWidgets.QListWidget(self.uiNIOVDEListGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIOVDEListWidget.sizePolicy().hasHeightForWidth())
        self.uiNIOVDEListWidget.setSizePolicy(sizePolicy)
        self.uiNIOVDEListWidget.setObjectName("uiNIOVDEListWidget")
        self.vboxlayout5.addWidget(self.uiNIOVDEListWidget)
        self.gridlayout9.addWidget(self.uiNIOVDEListGroupBox, 0, 2, 3, 1)
        self.uiAddNIOVDEPushButton = QtWidgets.QPushButton(self.NIOVDETab)
        self.uiAddNIOVDEPushButton.setObjectName("uiAddNIOVDEPushButton")
        self.gridlayout9.addWidget(self.uiAddNIOVDEPushButton, 1, 0, 1, 1)
        self.uiDeleteNIOVDEPushButton = QtWidgets.QPushButton(self.NIOVDETab)
        self.uiDeleteNIOVDEPushButton.setEnabled(False)
        self.uiDeleteNIOVDEPushButton.setObjectName("uiDeleteNIOVDEPushButton")
        self.gridlayout9.addWidget(self.uiDeleteNIOVDEPushButton, 1, 1, 1, 1)
        spacerItem7 = QtWidgets.QSpacerItem(161, 201, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.gridlayout9.addItem(spacerItem7, 2, 0, 2, 2)
        spacerItem8 = QtWidgets.QSpacerItem(196, 132, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridlayout9.addItem(spacerItem8, 3, 2, 1, 1)
        self.uiNIOsTabWidget.addTab(self.NIOVDETab, "")
        self.NIONullTab = QtWidgets.QWidget()
        self.NIONullTab.setObjectName("NIONullTab")
        self.gridlayout13 = QtWidgets.QGridLayout(self.NIONullTab)
        self.gridlayout13.setObjectName("gridlayout13")
        self.uiNIONullSettingsGroupBox = QtWidgets.QGroupBox(self.NIONullTab)
        self.uiNIONullSettingsGroupBox.setObjectName("uiNIONullSettingsGroupBox")
        self.gridlayout14 = QtWidgets.QGridLayout(self.uiNIONullSettingsGroupBox)
        self.gridlayout14.setObjectName("gridlayout14")
        self.uiNIONullIdentifierLabel = QtWidgets.QLabel(self.uiNIONullSettingsGroupBox)
        self.uiNIONullIdentifierLabel.setObjectName("uiNIONullIdentifierLabel")
        self.gridlayout14.addWidget(self.uiNIONullIdentifierLabel, 0, 0, 1, 1)
        self.uiNIONullIdentiferLineEdit = QtWidgets.QLineEdit(self.uiNIONullSettingsGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIONullIdentiferLineEdit.sizePolicy().hasHeightForWidth())
        self.uiNIONullIdentiferLineEdit.setSizePolicy(sizePolicy)
        self.uiNIONullIdentiferLineEdit.setObjectName("uiNIONullIdentiferLineEdit")
        self.gridlayout14.addWidget(self.uiNIONullIdentiferLineEdit, 1, 0, 1, 1)
        self.gridlayout13.addWidget(self.uiNIONullSettingsGroupBox, 0, 0, 1, 2)
        self.uiNIONullListGroupBox = QtWidgets.QGroupBox(self.NIONullTab)
        self.uiNIONullListGroupBox.setObjectName("uiNIONullListGroupBox")
        self.vboxlayout6 = QtWidgets.QVBoxLayout(self.uiNIONullListGroupBox)
        self.vboxlayout6.setObjectName("vboxlayout6")
        self.uiNIONullListWidget = QtWidgets.QListWidget(self.uiNIONullListGroupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.uiNIONullListWidget.sizePolicy().hasHeightForWidth())
        self.uiNIONullListWidget.setSizePolicy(sizePolicy)
        self.uiNIONullListWidget.setObjectName("uiNIONullListWidget")
        self.vboxlayout6.addWidget(self.uiNIONullListWidget)
        self.gridlayout13.addWidget(self.uiNIONullListGroupBox, 0, 2, 3, 1)
        self.uiAddNIONullPushButton = QtWidgets.QPushButton(self.NIONullTab)
        self.uiAddNIONullPushButton.setObjectName("uiAddNIONullPushButton")
        self.gridlayout13.addWidget(self.uiAddNIONullPushButton, 1, 0, 1, 1)
        self.uiDeleteNIONullPushButton = QtWidgets.QPushButton(self.NIONullTab)
        self.uiDeleteNIONullPushButton.setEnabled(False)
        self.uiDeleteNIONullPushButton.setObjectName("uiDeleteNIONullPushButton")
        self.gridlayout13.addWidget(self.uiDeleteNIONullPushButton, 1, 1, 1, 1)
        spacerItem9 = QtWidgets.QSpacerItem(20, 261, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridlayout13.addItem(spacerItem9, 2, 0, 2, 2)
        spacerItem10 = QtWidgets.QSpacerItem(20, 181, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridlayout13.addItem(spacerItem10, 3, 2, 1, 1)
        self.uiNIOsTabWidget.addTab(self.NIONullTab, "")
        self.MiscTab = QtWidgets.QWidget()
        self.MiscTab.setObjectName("MiscTab")
        self.gridLayout = QtWidgets.QGridLayout(self.MiscTab)
        self.gridLayout.setObjectName("gridLayout")
        self.uiNameLabel = QtWidgets.QLabel(self.MiscTab)
        self.uiNameLabel.setObjectName("uiNameLabel")
        self.gridLayout.addWidget(self.uiNameLabel, 0, 0, 1, 1)
        self.uiNameLineEdit = QtWidgets.QLineEdit(self.MiscTab)
        self.uiNameLineEdit.setObjectName("uiNameLineEdit")
        self.gridLayout.addWidget(self.uiNameLineEdit, 0, 1, 1, 1)
        spacerItem11 = QtWidgets.QSpacerItem(20, 399, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.gridLayout.addItem(spacerItem11, 1, 1, 1, 1)
        self.uiNIOsTabWidget.addTab(self.MiscTab, "")
        self.vboxlayout.addWidget(self.uiNIOsTabWidget)

        self.retranslateUi(cloudConfigPageWidget)
        self.uiNIOsTabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(cloudConfigPageWidget)

    def retranslateUi(self, cloudConfigPageWidget):
        _translate = QtCore.QCoreApplication.translate
        cloudConfigPageWidget.setWindowTitle(_translate("cloudConfigPageWidget", "Cloud configuration"))
        self.uiGenericEthernetGroupBox.setTitle(_translate("cloudConfigPageWidget", "Generic Ethernet NIO"))
        self.uiAddGenericEthernetPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteGenericEthernetPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiLinuxEthernetGroupBox.setTitle(_translate("cloudConfigPageWidget", "Linux Ethernet NIO (Linux only)"))
        self.uiAddLinuxEthernetPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteLinuxEthernetPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIOEthernetTab), _translate("cloudConfigPageWidget", "Ethernet"))
        self.uiNIONATSettingsGroupBox.setTitle(_translate("cloudConfigPageWidget", "Settings"))
        self.uiNIONATIdentifierLabel.setText(_translate("cloudConfigPageWidget", "Local identifier:"))
        self.uiNIONATListGroupBox.setTitle(_translate("cloudConfigPageWidget", "NIOs"))
        self.uiAddNIONATPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIONATPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIONATTab), _translate("cloudConfigPageWidget", "NAT"))
        self.uiNIOUDPSettingsGroupBox.setTitle(_translate("cloudConfigPageWidget", "Settings"))
        self.uiLocalPortLabel.setText(_translate("cloudConfigPageWidget", "Local port:"))
        self.uiRemoteHostLabel.setText(_translate("cloudConfigPageWidget", "Remote host:"))
        self.uiRemoteHostLineEdit.setText(_translate("cloudConfigPageWidget", "127.0.0.1"))
        self.uiRemotePortLabel.setText(_translate("cloudConfigPageWidget", "Remote port:"))
        self.uiNIOUDPListGroupBox.setTitle(_translate("cloudConfigPageWidget", "NIOs"))
        self.uiAddNIOUDPPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIOUDPPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIOUDPTab), _translate("cloudConfigPageWidget", "UDP"))
        self.uiNIOTAPGroupBox.setTitle(_translate("cloudConfigPageWidget", "TAP interface"))
        self.uiAddNIOTAPPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIOTAPPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIOTAPTab), _translate("cloudConfigPageWidget", "TAP"))
        self.uiNIOUNIXSettingsGroupBox.setTitle(_translate("cloudConfigPageWidget", "Settings"))
        self.uiLocalFileLabel.setText(_translate("cloudConfigPageWidget", "Local file:"))
        self.uiRemoteFileLabel.setText(_translate("cloudConfigPageWidget", "Remote file:"))
        self.uiNIOUNIXListGroupBox.setTitle(_translate("cloudConfigPageWidget", "NIOs"))
        self.uiAddNIOUNIXPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIOUNIXPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIOUnixTab), _translate("cloudConfigPageWidget", "UNIX"))
        self.uiNIOVDESettingsGroupBox.setTitle(_translate("cloudConfigPageWidget", "Settings"))
        self.uiVDEControlFileLabel.setText(_translate("cloudConfigPageWidget", "Control file:"))
        self.uiVDELocalFileLabel.setText(_translate("cloudConfigPageWidget", "Local file:"))
        self.uiNIOVDEListGroupBox.setTitle(_translate("cloudConfigPageWidget", "NIOs"))
        self.uiAddNIOVDEPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIOVDEPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIOVDETab), _translate("cloudConfigPageWidget", "VDE"))
        self.uiNIONullSettingsGroupBox.setTitle(_translate("cloudConfigPageWidget", "Settings"))
        self.uiNIONullIdentifierLabel.setText(_translate("cloudConfigPageWidget", "Local identifier:"))
        self.uiNIONullListGroupBox.setTitle(_translate("cloudConfigPageWidget", "NIOs"))
        self.uiAddNIONullPushButton.setText(_translate("cloudConfigPageWidget", "&Add"))
        self.uiDeleteNIONullPushButton.setText(_translate("cloudConfigPageWidget", "&Delete"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.NIONullTab), _translate("cloudConfigPageWidget", "NULL"))
        self.uiNameLabel.setText(_translate("cloudConfigPageWidget", "Name:"))
        self.uiNIOsTabWidget.setTabText(self.uiNIOsTabWidget.indexOf(self.MiscTab), _translate("cloudConfigPageWidget", "Misc."))




#!/usr/bin/env python
#
# Copyright (C) 2015 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


from ..servers import Servers
from ..qt import QtWidgets


def server_select(parent, allow_local_server=True):
    """
    Show a popup asking user to choose a server

    If only local server is available return it by default

    :params parent: Parent window
    :params allow_local_server: Boolean Is local server allowed
    :returns: Server or None
    """

    servers = Servers.instance()
    local_server = servers.localServer()
    remote_servers = servers.remoteServers()
    gns3_vm = Servers.instance().vmServer()

    if allow_local_server:
        server_list = ["Local server ({})".format(local_server.url())]
    else:
        server_list = []

    if gns3_vm:
        server_list.append("GNS3 VM ({})".format(gns3_vm.url()))
    for remote_server in remote_servers.values():
        server_list.append("{}".format(remote_server.url()))

    if len(server_list) == 0:
        raise ValueError("No server available")
    elif len(server_list) == 1:
        selection = server_list[0]
    else:
        (selection, ok) = QtWidgets.QInputDialog.getItem(parent, "Server", "Please choose a server", server_list, 0, False)
        if not ok:
            return None

    if selection.startswith("Local server"):
        return local_server
    elif selection.startswith("GNS3 VM"):
        return gns3_vm
    else:
        for server in remote_servers.values():
            if selection == server.url():
                return server



# -*- coding: utf-8 -*-
#
# Copyright (C) 2014 GNS3 Technologies Inc.
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
Nodes view that list all the available nodes to be dragged and dropped on the QGraphics scene.
"""

import pickle
from .qt import QtCore, QtGui, QtWidgets, qpartial
from .qt.qimage_svg_renderer import QImageSvgRenderer
from .modules import MODULES
from .node import Node
from .dialogs.configuration_dialog import ConfigurationDialog


class NodesView(QtWidgets.QTreeWidget):

    """
    Nodes view to list the nodes.

    :param parent: parent widget
    """

    def __init__(self, parent=None):

        super().__init__(parent)
        self._current_category = None

        # enables the possibility to drag items.
        self.setDragEnabled(True)

    def refresh(self):
        self.clear()
        self.populateNodesView(self._current_category)

    def populateNodesView(self, category):
        """
        Populates the nodes view with the device list of the specified
        category (None = all devices).

        :param category: category of device to list
        """

        self._current_category = category
        for module in MODULES:
            for node in module.instance().nodes():
                if category is not None and category not in node["categories"]:
                    continue
                item = QtWidgets.QTreeWidgetItem(self)
                item.setText(0, node["name"])
                item.setData(0, QtCore.Qt.UserRole, node)
                image = QtGui.QImage(32, 32, QtGui.QImage.Format_ARGB32)
                # Set the ARGB to 0 to prevent rendering artifacts
                image.fill(0x00000000)
                svg_renderer = QImageSvgRenderer(node["symbol"])
                svg_renderer.render(QtGui.QPainter(image))
                icon = QtGui.QIcon(QtGui.QPixmap.fromImage(image))
                item.setIcon(0, icon)

        self.sortByColumn(0, QtCore.Qt.AscendingOrder)
        if not self.topLevelItemCount() and category == Node.routers:
            QtWidgets.QMessageBox.warning(self, 'Routers', 'No routers have been configured.<br>You must provide your own router images in order to use GNS3.<br><br><a href="https://gns3.com/support/docs/adding-ios-or-iou-qemu-virtual-2">Show documentation</a>')
            # TODO: would be nicer to use QErrorMessage but the link cannot be clicked by default
            #QtWidgets.QErrorMessage.qtHandler().showMessage('No routers have been configured.<br>You must provide your own router images in order to use GNS3.<br><br><a href="https://community.gns3.com/community/software/documentation">Show documentation</a>')

    def mousePressEvent(self, event):
        """
        Handles all mouse press events.

        :param: QMouseEvent instance
        """

        # Check that an item has been selected and right click
        if self.currentItem() is not None and event.button() == QtCore.Qt.RightButton:
            self._showContextualMenu()
            event.accept()
            return
        super().mousePressEvent(event)

    def mouseMoveEvent(self, event):
        """
        Handles all mouse move events.
        This is the starting point to drag & drop a node on the scene.

        :param: QMouseEvent instance
        """

        # Check that an item has been selected and left button clicked
        if self.currentItem() is not None and event.buttons() == QtCore.Qt.LeftButton:
            item = self.currentItem()
            icon = item.icon(0)

            # retrieve the node class from the item data
            node = item.data(0, QtCore.Qt.UserRole)
            mimedata = QtCore.QMimeData()

            # pickle the node class, set the Mime type and data
            # and start dragging the item.
            data = pickle.dumps(node)
            mimedata.setData("application/x-gns3-node", data)
            drag = QtGui.QDrag(self)
            drag.setMimeData(mimedata)
            drag.setPixmap(icon.pixmap(self.iconSize()))
            drag.setHotSpot(QtCore.QPoint(drag.pixmap().width(), drag.pixmap().height()))
            drag.exec_(QtCore.Qt.CopyAction)
            event.accept()

    def _showContextualMenu(self):
        item = self.currentItem()
        node = item.data(0, QtCore.Qt.UserRole)
        node_module = None
        for module in MODULES:
            node_class = module.getNodeClass(node["class"])
            if node_class:
                break

        # We can not edit stuff like EthernetSwitch
        # or without config template like VPCS
        if not "builtin" in node and hasattr(module, "vmConfigurationPage"):
            for vm_key, vm in module.instance().VMs().items():
                if vm["name"] == node["name"]:
                    break
            menu = QtWidgets.QMenu()
            configuration = QtWidgets.QAction("Configure Template", menu)
            configuration.setIcon(QtGui.QIcon(":/icons/configuration.svg"))
            configuration.triggered.connect(qpartial(self._configurationSlot, vm, module))
            menu.addAction(configuration)

            configuration = QtWidgets.QAction("Delete Template", menu)
            configuration.setIcon(QtGui.QIcon(":/icons/delete.svg"))
            configuration.triggered.connect(qpartial(self._deleteSlot, vm_key, vm, module))
            menu.addAction(configuration)

            menu.exec_(QtGui.QCursor.pos())

    def _configurationSlot(self, vm, module, source):

        dialog = ConfigurationDialog(vm["name"], vm, module.vmConfigurationPage()(), parent=self)
        dialog.show()
        if dialog.exec_():
            module.instance().setVMs(module.instance().VMs())
            self.refresh()

    def _deleteSlot(self, vm_key, vm, module, source):

        reply = QtWidgets.QMessageBox.question(self, "Template", "Delete {} template?".format(vm["name"]),
                                               QtWidgets.QMessageBox.Yes, QtWidgets.QMessageBox.No)
        if reply == QtWidgets.QMessageBox.Yes:
            vms = module.instance().VMs()
            vms.pop(vm_key)
            module.instance().setVMs(vms)
            self.refresh()
